!=========================================================================
! Copyright (C) VMware, Inc. 1986-2011.  All Rights Reserved.
!
! $Id: btreebasicleaf.gs,v 1.7 2008-01-09 19:02:29 stever Exp $
!
! Superclass Hierarchy:
!   BtreeBasicLeafNode, BtreeLeafNode, BtreeNode, Array,
!   SequenceableCollection, Collection, Object.
!
!=========================================================================

! class created in idxclasses.topaz

removeallmethods BtreeBasicLeafNode
removeallclassmethods BtreeBasicLeafNode

protectmethods

category: 'For Documentation Installation only'
classmethod: BtreeBasicLeafNode
installDocumentation

< unprotected >

| doc txt |
doc := GsClassDocumentation _newForPrivateGsClass: self.

txt := (GsDocText new) details:
'A BtreeBasicLeafNode represents an exterior node of a B-tree for which a binary
 search can be done for a key.' .
doc documentClassWith: txt.

self description: doc.
%

category: 'Searching'
method: BtreeBasicLeafNode
_binarySearchCoveringKey: aKey totalOrder: aBoolean 

"Returns the index for the first entry in which aKey is found utilizing a
 binary search.  This is the first entry whose key >= aKey."

<protected primitive: 114>
self _primitiveFailed: #_binarySearchCoveringKey:totalOrder: .
self _uncontinuableError
%

category: 'Searching'
method: BtreeBasicLeafNode
_binarySearchCoveringKey: aKey value: aValue 

"Returns the index for the entry in which aKey/aValue would be inserted.  This
 is the first entry which has its key > aKey.  If multiple entries have the
 same key, then aValue is used (since entries with duplicate keys are inserted
 by the value's OOP ordering).  If the receiver is empty, returns 1.  Uses a
 binary search."

<protected primitive: 112>
self _primitiveFailed: #_binarySearchCoveringKey:value: .
self _uncontinuableError
%

category: 'Searching'
method: BtreeBasicLeafNode
_binarySearchForKey: aKey value: aValue 

"Returns the 'value' index whose key equality matches aKey.  If no match is
 found, returns zero.  Uses a binary search."

<protected primitive: 113>
self _primitiveFailed: #_binarySearchForKey:value: .
self _uncontinuableError
%

category: 'Constants'
method: BtreeBasicLeafNode
_encryptionSize

"Three slots are used for encryption."
<protected>

^ 3
%

category: 'Updating'
method: BtreeBasicLeafNode
_insertEncryptionFor: aKey value: aValue startingAt: index

"Gets the encryption for the given value and places it in the receiver starting
 at the given index."

<protected primitive: 373>
self _primitiveFailed: #_insertEncryptionFor:value:startingAt: .
self _uncontinuableError
%

category: 'Constants'
method: BtreeBasicLeafNode
entrySize

"Returns the size of an entry (with 3 slots for encryption)."
<protected>

^ 5
%

category: 'Constants'
classmethod: BtreeBasicLeafNode
entrySize

"Returns the size of an entry (with 3 slots for encryption)."
<protected>

^ 5
%

category: 'Constants'
classmethod: BtreeBasicLeafNode
maxNumberOfElements

"Returns the number of entries that are allowed in a leaf node."
<protected>

^ 400
%

category: 'Constants'
method: BtreeBasicLeafNode
mergeThreshold

"Returns the threshold such that if an entry has less than this number, it must
 have elements added from its sibling or be merged with its sibling.  This
 value is usually the same as the result of the message
 'self maxNumberOfElements quo: 2'."
<protected>

^ 200
%

category: 'Constants'
method: BtreeBasicLeafNode
parentNodeClass

"Returns the class of node to be created as the parent when a split occurs."
<protected>

^ BtreeBasicInteriorNode
%

category: 'Testing'
method: BtreeBasicLeafNode
_canCompare: aKey withClass: aClass

"Returns whether the receiver can make comparisons with the given key."
<protected>

^ self _basicCanCompare: aKey withClass: aClass
%

category: 'Constants'
method: BtreeBasicLeafNode
_emptyEntryArray

"Returns a five-element Array of nils."
<protected>

^ #( nil nil nil nil nil)
%

category: 'Accessing'
method: BtreeBasicLeafNode
_lastIndex

"Returns the index of the last used slot in the node."
<protected>

^ numElements * 5
%

category: 'Accessing'
method: BtreeBasicLeafNode
_lastKeyIndex

"Returns the index of the key of the last entry, or zero if the node is empty."
<protected>

^ numElements == 0
    ifTrue: [ 0 ]
    ifFalse: [ numElements * 5 - 3 ]
%

category: 'Comparison Operators'
method: BtreeBasicLeafNode
_compareKey: aKey equalToEntryAt: index 

"Performs an equality comparison between aKey and the entry whose key is at the
 given index.  Use encryption to make the comparison faster."

<protected primitive: 160>
self _primitiveFailed: #_compareKey:equalToEntryAt: .
self _uncontinuableError
%

category: 'Testing'
method: BtreeBasicLeafNode
_isSymbolAt: offset

"Return whether the key at the given offset is a symbol.
This method uses the encryption to avoid fetching the object."

<protected primitive: 47>
self _primitiveFailed: #_isSymbolAt: .
self _uncontinuableError
%

unprotectmethods

